@@ -10,11 +10,9 @@ from icontract._globals import CallableT
 from icontract._types import Contract, Snapshot
 from icontract.errors import ViolationError
 
-# pylint: disable=protected-access
 # pylint does not play with typing.Mapping.
 # pylint: disable=unsubscriptable-object
 # pylint: disable=raising-bad-type
-# pylint: disable=too-many-lines
 
 
 def _walk_decorator_stack(func: CallableT) -> Iterable['CallableT']:
@@ -52,8 +50,6 @@ def kwargs_from_call(param_names: List[str], kwdefaults: Dict[str, Any], args: T
     :param kwargs: keyword arguments supplied to the call
     :return: resolved arguments as they would be passed to the function
     """
-    # pylint: disable=too-many-arguments
-
     # (Marko Ristin, 2020-12-01)
     # Insert _ARGS and _KWARGS preemptively even if they are not needed by any contract.
     # This makes the code logic much simpler since we do not explicitly check if a contract would
@@ -176,14 +172,13 @@ def _assert_resolved_kwargs_valid(postconditions: List[Contract],
     return None
 
 
-def _create_violation_error(contract: Contract, resolved_kwargs: Mapping[str, Any],
-                            condition_kwargs: Mapping[str, Any]) -> BaseException:
+def _create_violation_error(contract: Contract, resolved_kwargs: Mapping[str, Any]) -> BaseException:
     """Create the violation error based on the violated contract."""
     exception = None  # type: Optional[BaseException]
 
     if contract.error is None:
         try:
-            msg = icontract._represent.generate_message(contract=contract, condition_kwargs=condition_kwargs)
+            msg = icontract._represent.generate_message(contract=contract, resolved_kwargs=resolved_kwargs)
         except Exception as err:
             parts = ["Failed to recompute the values of the contract condition:\n"]
             if contract.location is not None:
@@ -215,7 +210,7 @@ def _create_violation_error(contract: Contract, resolved_kwargs: Mapping[str, An
                 "The exception class supplied in the contract's error {} is not a subclass of BaseException.".format(
                     contract.error))
 
-        msg = icontract._represent.generate_message(contract=contract, condition_kwargs=condition_kwargs)
+        msg = icontract._represent.generate_message(contract=contract, resolved_kwargs=resolved_kwargs)
         exception = contract.error(msg)
     elif isinstance(contract.error, BaseException):
         exception = contract.error
@@ -255,8 +250,7 @@ async def _assert_preconditions_async(preconditions: List[List[Contract]],
                     check = check_or_coroutine
 
             if not_check(check=check, contract=contract):
-                exception = _create_violation_error(
-                    contract=contract, resolved_kwargs=resolved_kwargs, condition_kwargs=condition_kwargs)
+                exception = _create_violation_error(contract=contract, resolved_kwargs=resolved_kwargs)
                 break
 
         # The group of preconditions was satisfied, no need to check the other groups.
@@ -293,8 +287,7 @@ def _assert_preconditions(preconditions: List[List[Contract]], resolved_kwargs:
                     contract.condition, func))
 
             if not_check(check=check, contract=contract):
-                exception = _create_violation_error(
-                    contract=contract, resolved_kwargs=resolved_kwargs, condition_kwargs=condition_kwargs)
+                exception = _create_violation_error(contract=contract, resolved_kwargs=resolved_kwargs)
                 break
 
         # The group of preconditions was satisfied, no need to check the other groups.
@@ -373,8 +366,7 @@ async def _assert_postconditions_async(postconditions: List[Contract],
                 check = check_or_coroutine
 
         if not_check(check=check, contract=contract):
-            exception = _create_violation_error(
-                contract=contract, resolved_kwargs=resolved_kwargs, condition_kwargs=condition_kwargs)
+            exception = _create_violation_error(contract=contract, resolved_kwargs=resolved_kwargs)
 
             return exception
 
@@ -401,8 +393,7 @@ def _assert_postconditions(postconditions: List[Contract], resolved_kwargs: Mapp
                 contract.condition, func))
 
         if not_check(check=check, contract=contract):
-            exception = _create_violation_error(
-                contract=contract, resolved_kwargs=resolved_kwargs, condition_kwargs=condition_kwargs)
+            exception = _create_violation_error(contract=contract, resolved_kwargs=resolved_kwargs)
 
             return exception
 
@@ -417,13 +408,7 @@ def _assert_invariant(contract: Contract, instance: Any) -> None:
         check = contract.condition()
 
     if not_check(check=check, contract=contract):
-        if 'self' in contract.condition_arg_set:
-            condition_kwargs = {"self": instance}
-        else:
-            condition_kwargs = dict()
-
-        raise _create_violation_error(
-            contract=contract, resolved_kwargs={'self': instance}, condition_kwargs=condition_kwargs)
+        raise _create_violation_error(contract=contract, resolved_kwargs={'self': instance})
 
 
 def select_capture_kwargs(a_snapshot: Snapshot, resolved_kwargs: Mapping[str, Any]) -> Mapping[str, Any]:
@@ -522,7 +507,6 @@ _IN_PROGRESS = threading.local()
 
 def decorate_with_checker(func: CallableT) -> CallableT:
     """Decorate the function with a checker that verifies the preconditions and postconditions."""
-    # pylint: disable=too-many-statements
     assert not hasattr(func, "__preconditions__"), \
         "Expected func to have no list of preconditions (there should be only a single contract checker per function)."
 
@@ -565,8 +549,6 @@ def decorate_with_checker(func: CallableT) -> CallableT:
 
         async def wrapper(*args, **kwargs):  # type: ignore
             """Wrap func by checking the preconditions and postconditions."""
-            # pylint: disable=too-many-branches,too-many-nested-blocks,too-many-locals
-
             kwargs_error = _assert_no_invalid_kwargs(kwargs)
             if kwargs_error:
                 raise kwargs_error
@@ -620,8 +602,6 @@ def decorate_with_checker(func: CallableT) -> CallableT:
 
         def wrapper(*args, **kwargs):  # type: ignore
             """Wrap func by checking the preconditions and postconditions."""
-            # pylint: disable=too-many-branches,too-many-nested-blocks,too-many-locals
-
             kwargs_error = _assert_no_invalid_kwargs(kwargs)
             if kwargs_error:
                 raise kwargs_error
@@ -758,7 +738,7 @@ def _find_self(param_names: List[str], args: Tuple[Any, ...], kwargs: Dict[str,
     except ValueError:
         pass
 
-    if instance_i is not None:
+    if instance_i is not None and instance_i < len(args):
         return args[instance_i]
 
     return kwargs["self"]
@@ -804,7 +784,6 @@ def _decorate_with_invariants(func: CallableT, is_init: bool) -> CallableT:
     :param is_init: True if the ``func`` is __init__
     :return: function wrapped with invariant checks
     """
-    # pylint: disable=too-many-statements
     if _already_decorated_with_invariants(func=func):
         return func
 
@@ -947,8 +926,6 @@ def _already_decorated_with_invariants(func: CallableT) -> bool:
 
 def add_invariant_checks(cls: type) -> None:
     """Decorate each of the class functions with invariant checks if not already decorated."""
-    # pylint: disable=too-many-branches
-
     # Candidates for the decoration as list of (name, dir() value)
     init_name_func = None  # type: Optional[Tuple[str, Callable[..., None]]]
     names_funcs = []  # type: List[Tuple[str, Callable[..., None]]]
